﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.TestFunctions.Evaluators {
  [Item("CustomEvaluator", "Evaluates a custom test fuction.")]
  [StorableClass]
  public class CustomEvaluator : SingleObjectiveTestFunctionProblemEvaluator {


    [StorableConstructor]
    protected CustomEvaluator(bool deserializing) : base(deserializing) { }
    protected CustomEvaluator(CustomEvaluator original, Cloner cloner) : base(original, cloner) { }
    public CustomEvaluator() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CustomEvaluator(this, cloner);
    }

    public override bool Maximization {
      get { return false; }
    }

    public override DoubleMatrix Bounds {
      get { return new DoubleMatrix(new double[,] { { 0, 5 } }); }
    }

    public override double BestKnownQuality {
      get {
        return -1;
      }
    }

    public override int MinimumProblemSize { get { return 1; } }

    public override int MaximumProblemSize { get { return 2; } }

    public override RealVector GetBestKnownSolution(int dimension) {
      return new RealVector(dimension);
    }

    public double Evaluate(RealVector point) {
      return EvaluateFunction(point);
    }

    protected override double EvaluateFunction(RealVector point) {
      if (point[0] < 0 || point[1] < 0)
        return 0;
      return Math.Sin(Math.Pow(point[0], point[1]));
    }
  }
}
